@@ -1,27 +1,3 @@
-nodes_arg_to_pts <- function (nodes, graph)
-{
-    if (!is.matrix (nodes))
-        nodes <- as.matrix (nodes)
-    if (ncol (nodes) == 2)
-    {
-        verts <- dodgr_vertices (graph)
-        nodes <- verts$id [match_pts_to_graph (verts, nodes)]
-    }
-    return (nodes)
-}
-
-
-# keep from and to routing points in contracted graph
-contract_graph_with_pts <- function (graph, from, to)
-{
-    pts <- NULL
-    if (!missing (from))
-        pts <- c (pts, from)
-    if (!missing (to))
-        pts <- c (pts, to)
-    dodgr_contract_graph (graph, unique (pts))
-}
-
 #' dodgr_flows_aggregate
 #'
 #' Aggregate flows throughout a network based on an input matrix of flows
@@ -40,7 +16,7 @@ contract_graph_with_pts <- function (graph, from, to)
 #' generally be much faster).
 #' @param heap Type of heap to use in priority queue. Options include
 #' Fibonacci Heap (default; `FHeap`), Binary Heap (`BHeap`),
-#' `Radix`, Trinomial Heap (`TriHeap`), Extended Trinomial Heap
+#' Trinomial Heap (`TriHeap`), Extended Trinomial Heap
 #' (`TriHeapExt`, and 2-3 Heap (`Heap23`).
 #' @param tol Relative tolerance below which flows towards `to` vertices are not
 #' considered. This will generally have no effect, but can provide speed gains
@@ -48,8 +24,43 @@ contract_graph_with_pts <- function (graph, from, to)
 #' parameter effectively reduces the radius from each `from` point over which
 #' flows are aggregated. To remove any such effect, set `tol = 0`.
 #' @param quiet If `FALSE`, display progress messages on screen.
+#' @inheritParams dodgr_flows_si
 #' @return Modified version of graph with additonal `flow` column added.
 #'
+#' @note Spatial Interaction models are often fitted through trialling a range
+#' of values of 'k'. The specification above allows fitting multiple values of
+#' 'k' to be done with a single call, in a way that is far more efficient than
+#' making multiple calls. A matrix of 'k' values may be entered, with each
+#' column holding a different vector of values, one for each 'from' point. For a
+#' matrix of 'k' values having 'n' columns, the return object will be a modified
+#' version in the input 'graph', with an additional 'n' columns, named 'flow1',
+#' 'flow2', ... up to 'n'. These columns must be subsequently matched by the
+#' user back on to the corresponding columns of the matrix of 'k' values.
+#'
+#' @note The `norm_sums` parameter should be used whenever densities at origins
+#' and destinations are absolute values, and ensures that the sum of resultant
+#' flow values throughout the entire network equals the sum of densities at all
+#' origins. For example, with `norm_sums = TRUE` (the default), a flow from a
+#' single origin with density one to a single destination along two edges will
+#' allocate flows of one half to each of those edges, such that the sum of flows
+#' across the network will equal one, or the sum of densities from all origins.
+#' The `norm_sums = TRUE` option is appropriate where densities are relative
+#' values, and ensures that each edge maintains relative proportions. In the
+#' above example, flows along each of two edges would equal one, for a network
+#' sum of two, or greater than the sum of densities.
+#'
+#' With `norm_sums = TRUE`, the sum of network flows (`sum(output$flow)`) should
+#' equal the sum of origin densities (`sum(dens_from)`). This may nevertheless
+#' not always be the case, because origin points may simply be too far from any
+#' denstination (`to`) points for an exponential model to yield non-zero values
+#' anywhere in a network within machine tolerance. Such cases may result in sums
+#' of output flows being less than sums of input densities.
+#'
+#' Flows are calculated by default using parallel computation with the maximal
+#' number of available cores or threads. This number can be reduced by
+#' specifying a value via
+#' `RcppParallel::setThreadOptions (numThreads = <desired_number>)`.
+#'
 #' @examples
 #' graph <- weight_streetnet (hampi)
 #' from <- sample (graph$from_id, size = 10)
@@ -58,20 +69,17 @@ contract_graph_with_pts <- function (graph, from, to)
 #' flows <- matrix (10 * runif (length (from) * length (to)),
 #'                  nrow = length (from))
 #' graph <- dodgr_flows_aggregate (graph, from = from, to = to, flows = flows)
-#' # graph then has an additonal 'flows` column of aggregate flows along all
+#' # graph then has an additonal 'flows' column of aggregate flows along all
 #' # edges. These flows are directed, and can be aggregated to equivalent
 #' # undirected flows on an equivalent undirected graph with:
-#' graph_undir <- merge_directed_flows (graph)
+#' graph_undir <- merge_directed_graph (graph)
 #' # This graph will only include those edges having non-zero flows, and so:
 #' nrow (graph); nrow (graph_undir) # the latter is much smaller
 #'
 #' # The following code can be used to convert the resultant graph to an `sf`
 #' # object suitable for plotting
 #' \dontrun{
-#' geoms <- dodgr_to_sfc (graph_undir)
-#' gc <- dodgr_contract_graph (graph_undir)
-#' gsf <- sf::st_sf (geoms)
-#' gsf$flow <- gc$flow
+#' gsf <- dodgr_to_sf (graph_undir)
 #'
 #' # example of plotting with the 'mapview' package
 #' library (mapview)
@@ -106,9 +114,10 @@ contract_graph_with_pts <- function (graph, from, to)
 #' r$id <- seq (nrow (r))
 #' graph <- weight_streetnet (r, type_col = "type", id_col = "id",
 #'                            wt_profile = 1)
-#' f <- dodgr_flows_aggregate (graph, from = xy_start, to = xy_end, flows = flows)
+#' f <- dodgr_flows_aggregate (graph, from = xy_start, to = xy_end,
+#'                             flows = flows)
 #' # Then merge directed flows and convert to \pkg{sf} for plotting as before:
-#' f <- merge_directed_flows (f)
+#' f <- merge_directed_graph (f)
 #' geoms <- dodgr_to_sfc (f)
 #' gc <- dodgr_contract_graph (f)
 #' gsf <- sf::st_sf (geoms)
@@ -118,12 +127,9 @@ contract_graph_with_pts <- function (graph, from, to)
 #' }
 #' @export
 dodgr_flows_aggregate <- function (graph, from, to, flows, contract = FALSE,
-                                   heap = "BHeap", tol = 1e-12, quiet = TRUE)
+                                   heap = "BHeap", tol = 1e-12,
+                                   norm_sums = TRUE, quiet = TRUE)
 {
-    if ("flow" %in% names (graph))
-        warning ("graph already has a 'flow' column; ",
-                  "this will be overwritten")
-
     if (any (is.na (flows))) {
         flows [is.na (flows)] <- 0
     }
@@ -131,28 +137,6 @@ dodgr_flows_aggregate <- function (graph, from, to, flows, contract = FALSE,
     heap <- hps$heap
     graph <- hps$graph
 
-    gr_cols <- dodgr_graph_cols (graph)
-
-    # change from and to just to check conformity
-    tp <- attr (graph, "turn_penalty")
-    tp <- ifelse (is.null (tp), 0, tp)
-    if (!missing (from))
-    {
-        # remove any routing points not in edge start nodes:
-        from <- nodes_arg_to_pts (from, graph)
-        if (is (graph, "dodgr_streetnet_sc") & tp > 0)
-            from <- remap_verts_with_turn_penalty (graph, from, from = TRUE)
-        from <- from [from %in% graph [[gr_cols$from]] ]
-    }
-    if (!missing (to))
-    {
-        # remove any routing points not in edge end nodes:
-        to <- nodes_arg_to_pts (to, graph)
-        if (is (graph, "dodgr_streetnet_sc") & tp > 0)
-            to <- remap_verts_with_turn_penalty (graph, to, from = FALSE)
-        to <- to [to %in% graph [[gr_cols$to]] ]
-    }
-
     if (contract)
     {
         graph_full <- graph
@@ -165,28 +149,16 @@ dodgr_flows_aggregate <- function (graph, from, to, flows, contract = FALSE,
         edge_map <- readRDS (fname_c)
     }
 
-    vert_map <- make_vert_map (graph, gr_cols)
-
-    index_id <- get_index_id_cols (graph, gr_cols, vert_map, from)
-    from_index <- index_id$index - 1 # 0-based
-    index_id <- get_index_id_cols (graph, gr_cols, vert_map, to)
-    to_index <- index_id$index - 1 # 0-based
-
+    g <- prepare_graph (graph, from, to)
     if (!is.matrix (flows))
-        flows <- matrix (flows, nrow = length (from_index))
-
-    graph2 <- convert_graph (graph, gr_cols)
+        flows <- matrix (flows, nrow = length (g$from_index))
 
     if (!quiet)
         message ("\nAggregating flows ... ", appendLF = FALSE)
 
-    dirtxt <- get_random_prefix ()
-    rcpp_flows_aggregate_par (graph2, vert_map, from_index, to_index,
-                              flows, tol, dirtxt, heap)
-    f <- list.files (tempdir (), full.names = TRUE)
-    files <- f [grep (dirtxt, f)]
-    graph$flow <- rcpp_aggregate_files (files, nrow (graph))
-    junk <- file.remove (files) # nolint
+    graph$flow <- rcpp_flows_aggregate_par (g$graph, g$vert_map,
+                                            g$from_index, g$to_index,
+                                            flows, norm_sums, tol, heap)
 
     if (contract) # map contracted flows back onto full graph
         graph <- uncontract_graph (graph, edge_map, graph_full)
@@ -194,18 +166,12 @@ dodgr_flows_aggregate <- function (graph, from, to, flows, contract = FALSE,
     return (graph)
 }
 
-get_random_prefix <- function (n = 5)
-{
-    charvec <- c (letters, LETTERS, 0:9)
-    prefix <- paste0 (sample (charvec, n, replace = TRUE), collapse = "")
-    file.path (tempdir (), paste0 ("flow_", prefix))
-}
-
 #' dodgr_flows_disperse
 #'
 #' Disperse flows throughout a network based on a input vectors of origin points
 #' and associated densities
 #'
+#' @inheritParams dodgr_flows_aggregate
 #' @param graph `data.frame` or equivalent object representing the network
 #' graph (see Details)
 #' @param from Vector or matrix of points **from** which aggregate dispersed
@@ -216,19 +182,21 @@ get_random_prefix <- function (n = 5)
 #' also be a vector with same length as `from`, giving dispersal coefficients
 #' from each point. If value of `k<0` is given, a standard logistic polynomial
 #' will be used.
-#' @param contract If `TRUE`, calculate flows on contracted graph before
-#' mapping them back on to the original full graph (recommended as this will
-#' generally be much faster).
-#' @param heap Type of heap to use in priority queue. Options include
-#' Fibonacci Heap (default; `FHeap`), Binary Heap (`BHeap`),
-#' `Radix`, Trinomial Heap (`TriHeap`), Extended Trinomial Heap
-#' (`TriHeapExt`, and 2-3 Heap (`Heap23`).
 #' @param tol Relative tolerance below which dispersal is considered to have
 #' finished. This parameter can generally be ignored; if in doubt, its effect
 #' can be removed by setting `tol = 0`.
-#' @param quiet If `FALSE`, display progress messages on screen.
 #' @return Modified version of graph with additonal `flow` column added.
 #'
+#' @note Spatial Interaction models are often fitted through trialling a range
+#' of values of 'k'. The specification above allows fitting multiple values of
+#' 'k' to be done with a single call, in a way that is far more efficient than
+#' making multiple calls. A matrix of 'k' values may be entered, with each
+#' column holding a different vector of values, one for each 'from' point. For a
+#' matrix of 'k' values having 'n' columns, the return object will be a modified
+#' version in the input 'graph', with an additional 'n' columns, named 'flow1',
+#' 'flow2', ... up to 'n'. These columns must be subsequently matched by the
+#' user back on to the corresponding columns of the matrix of 'k' values.
+#'
 #' @export
 #' @examples
 #' graph <- weight_streetnet (hampi)
@@ -238,40 +206,22 @@ get_random_prefix <- function (n = 5)
 #' # graph then has an additonal 'flows` column of aggregate flows along all
 #' # edges. These flows are directed, and can be aggregated to equivalent
 #' # undirected flows on an equivalent undirected graph with:
-#' graph_undir <- merge_directed_flows (graph)
-dodgr_flows_disperse <- function (graph, from, dens, k = 500, contract = FALSE, 
+#' graph_undir <- merge_directed_graph (graph)
+dodgr_flows_disperse <- function (graph, from, dens, k = 500, contract = FALSE,
                                   heap = 'BHeap', tol = 1e-12, quiet = TRUE)
 {
-    if ("flow" %in% names (graph))
-        warning ("graph already has a 'flow' column; ",
-                  "this will be overwritten")
-
-    if (!(length (k) == 1 | length (k) == length (from)))
-        stop ("'k' must be either single value or vector ",
-              "of same length as 'from'")
-    if (length (k) == 1)
-        k <- rep (k, length (from))
+    res <- check_k (k, from)
+    k <- res$k
+    nk <- res$nk
 
     if (any (is.na (dens))) {
         dens [is.na (dens)] <- 0
     }
+
     hps <- get_heap (heap, graph)
     heap <- hps$heap
     graph <- hps$graph
 
-    gr_cols <- dodgr_graph_cols (graph)
-
-    tp <- attr (graph, "turn_penalty")
-    tp <- ifelse (is.null (tp), 0, tp)
-    if (!missing (from))
-    {
-        # remove any routing points not in edge start nodes:
-        from <- nodes_arg_to_pts (from, graph)
-        if (is (graph, "dodgr_streetnet_sc") & tp > 0)
-            from <- remap_verts_with_turn_penalty (graph, from, from = TRUE)
-        from <- from [from %in% graph [[gr_cols$from]] ]
-    }
-
     if (contract)
     {
         graph_full <- graph
@@ -284,27 +234,24 @@ dodgr_flows_disperse <- function (graph, from, dens, k = 500, contract = FALSE,
         edge_map <- readRDS (fname_c)
     }
 
-    vert_map <- make_vert_map (graph, gr_cols)
-
-    index_id <- get_index_id_cols (graph, gr_cols, vert_map, from)
-    from_index <- index_id$index - 1 # 0-based
+    g <- prepare_graph (graph, from)
 
     if (!is.matrix (dens))
         dens <- as.matrix (dens)
 
-    graph2 <- convert_graph (graph, gr_cols)
-
     if (!quiet)
         message ("\nAggregating flows ... ", appendLF = FALSE)
 
-    # parallel results are dumped in tempdir
-    dirtxt <- get_random_prefix ()
-    rcpp_flows_disperse_par (graph2, vert_map, from_index,
-                             k, dens, tol, dirtxt, heap)
-    f <- list.files (tempdir (), full.names = TRUE)
-    files <- f [grep (dirtxt, f)]
-    graph$flow <- rcpp_aggregate_files (files, nrow (graph))
-    junk <- file.remove (files) # nolint
+    f <- rcpp_flows_disperse_par (g$graph, g$vert_map, g$from_index,
+                                  k, dens, tol, heap)
+    if (nk == 1)
+        graph$flow <- f
+    else
+    {
+        flowmat <- data.frame (matrix (f, ncol = nk))
+        names (flowmat) <- paste0 ("flow", seq (nk))
+        graph <- cbind (graph, flowmat)
+    }
 
     if (contract) # map contracted flows back onto full graph
         graph <- uncontract_graph (graph, edge_map, graph_full)
@@ -312,22 +259,54 @@ dodgr_flows_disperse <- function (graph, from, dens, k = 500, contract = FALSE,
     return (graph)
 }
 
-#' merge_directed_flows
+#' dodgr_flows_si
 #'
-#' The \link{dodgr_flows_aggregate} and \link{dodgr_flows_disperse} functions
-#' return a column of aggregated flows directed along each edge of a graph, so
-#' the aggregated flow from vertex A to vertex B will not necessarily equal that
-#' from B to A, and the total flow in both directions will be the sum of flow
-#' from A to B plus that from B to A. This function converts a directed graph to
-#' undirected form through reducing all pairs of directed edges to a single
-#' edge, and aggregating flows from both directions.
+#' Aggregate flows throughout a network based using an exponential Spatial
+#' Interaction (SI) model between a specified set of origin and destination
+#' points, and associated vectors of densities.
+#'
+#' @inheritParams dodgr_flows_aggregate
+#' @param k Width of exponential spatial interaction function (exp (-d / k)),
+#' in units of 'd', specified in one of 3 forms: (i) a single vlaue; (ii) a
+#' vector of independent values for each origin point (with same length as
+#' 'from' points); or (iii) an equivalent matrix with each column holding values
+#' for each 'from' point, so 'nrow(k)==length(from)'. See Note.
+#' @param dens_from Vector of densities at origin ('from') points
+#' @param dens_to Vector of densities at destination ('to') points
+#' @param norm_sums Standardise sums from all origin points, so sum of flows
+#' throughout entire network equals sum of densities from all origins (see
+#' Note).
+#' @return Modified version of graph with additonal `flow` column added.
+#'
+#' @note Spatial Interaction models are often fitted through trialling a range
+#' of values of 'k'. The specification above allows fitting multiple values of
+#' 'k' to be done with a single call, in a way that is far more efficient than
+#' making multiple calls. A matrix of 'k' values may be entered, with each
+#' column holding a different vector of values, one for each 'from' point. For a
+#' matrix of 'k' values having 'n' columns, the return object will be a modified
+#' version in the input 'graph', with an additional 'n' columns, named 'flow1',
+#' 'flow2', ... up to 'n'. These columns must be subsequently matched by the
+#' user back on to the corresponding columns of the matrix of 'k' values.
+#'
+#' @note The `norm_sums` parameter should be used whenever densities at origins
+#' and destinations are absolute values, and ensures that the sum of resultant
+#' flow values throughout the entire network equals the sum of densities at all
+#' origins. For example, with `norm_sums = TRUE` (the default), a flow from a
+#' single origin with density one to a single destination along two edges will
+#' allocate flows of one half to each of those edges, such that the sum of flows
+#' across the network will equal one, or the sum of densities from all origins.
+#' The `norm_sums = TRUE` option is appropriate where densities are relative
+#' values, and ensures that each edge maintains relative proportions. In the
+#' above example, flows along each of two edges would equal one, for a network
+#' sum of two, or greater than the sum of densities.
+#'
+#' With `norm_sums = TRUE`, the sum of network flows (`sum(output$flow)`) should
+#' equal the sum of origin densities (`sum(dens_from)`). This may nevertheless
+#' not always be the case, because origin points may simply be too far from any
+#' denstination (`to`) points for an exponential model to yield non-zero values
+#' anywhere in a network within machine tolerance. Such cases may result in sums
+#' of output flows being less than sums of input densities.
 #'
-#' @param graph A graph containing a `flow` column as returned from
-#' \link{dodgr_flows_aggregate} or \link{dodgr_flows_disperse}
-#' @return An equivalent graph in which all directed edges have been reduced to
-#' single, undirected edges, and all directed flows aggregated to undirected
-#' flows.
-#' @export
 #' @examples
 #' graph <- weight_streetnet (hampi)
 #' from <- sample (graph$from_id, size = 10)
@@ -336,29 +315,162 @@ dodgr_flows_disperse <- function (graph, from, dens, k = 500, contract = FALSE,
 #' flows <- matrix (10 * runif (length (from) * length (to)),
 #'                  nrow = length (from))
 #' graph <- dodgr_flows_aggregate (graph, from = from, to = to, flows = flows)
-#' # graph then has an additonal 'flows` column of aggregate flows along all
+#' # graph then has an additonal 'flows' column of aggregate flows along all
 #' # edges. These flows are directed, and can be aggregated to equivalent
 #' # undirected flows on an equivalent undirected graph with:
-#' graph_undir <- merge_directed_flows (graph)
+#' graph_undir <- merge_directed_graph (graph)
 #' # This graph will only include those edges having non-zero flows, and so:
 #' nrow (graph); nrow (graph_undir) # the latter is much smaller
-merge_directed_flows <- function (graph)
+#' @export
+dodgr_flows_si <- function (graph, from, to, k = 500, dens_from = NULL,
+                            dens_to = NULL, contract = FALSE,
+                            norm_sums = TRUE, heap = "BHeap",
+                            tol = 1e-12, quiet = TRUE)
 {
-    if (!"flow" %in% names (graph))
-        stop ("graph does not have any flows to merge")
+    if (missing (from))
+        stop ("'from' must be provided for spatial interaction models.")
 
-    gr_cols <- dodgr_graph_cols (graph)
-    graph2 <- convert_graph (graph, gr_cols)
-    graph2$flow <- graph$flow
+    hps <- get_heap (heap, graph)
+    heap <- hps$heap
+    graph <- hps$graph
 
-    flows <- rcpp_merge_flows (graph2)
+    res <- check_k (k, from)
+    k <- res$k
+    nk <- res$nk
 
-    indx <- which (flows > 0)
-    graph <- graph [indx, , drop = FALSE] #nolint
-    graph$flow <- flows [indx]
-    class (graph) <- c (class (graph), "dodgr_merged_flows")
+    if (contract)
+    {
+        graph_full <- graph
+        graph <- contract_graph_with_pts (graph, from, to)
+        hashc <- get_hash (graph, hash = FALSE)
+        fname_c <- file.path (tempdir (),
+                              paste0 ("dodgr_edge_map_", hashc, ".Rds"))
+        if (!file.exists (fname_c))
+            stop ("something went wrong extracting the edge_map ... ") # nocov
+        edge_map <- readRDS (fname_c)
+    }
 
-    attr (graph, "hash") <- digest::digest (graph [[gr_cols$edge_id]])
+    g <- prepare_graph (graph, from, to)
+
+    if (!quiet)
+        message ("\nAggregating flows ... ", appendLF = FALSE)
+
+    f <- rcpp_flows_si (g$graph, g$vert_map, g$from_index, g$to_index,
+                        k, dens_from, dens_to, norm_sums, tol, heap)
+
+    if (nk == 1)
+        graph$flow <- f
+    else
+    {
+        flowmat <- data.frame (matrix (f, ncol = nk))
+        names (flowmat) <- paste0 ("flow", seq (nk))
+        graph <- cbind (graph, flowmat)
+    }
+
+
+    if (contract) # map contracted flows back onto full graph
+        graph <- uncontract_graph (graph, edge_map, graph_full)
 
     return (graph)
 }
+
+check_k <- function (k, from)
+{
+    nk <- 1
+
+    if (is.data.frame (k))
+        k <- as.matrix (k)
+
+    if (is.matrix (k))
+    {
+        if (nrow (k) != length (from))
+            stop ("nrow(k) must equal length of 'from' points")
+        nk <- ncol (k)
+    } else if (is.numeric (k))
+    {
+        if (length (k) == 1)
+            k <- rep (k, length (from))
+        else if (length (k) != length (from))
+        {
+            # convert to matrix
+            nk <- length (k)
+            k <- array (rep (k, each = length (from)),
+                        dim = c (length (from), nk))
+        }
+    } else
+        stop ("'k' must be either a single value, a vector, or a matrix")
+
+    list (k = k, nk = nk)
+}
+
+# transform input graph and (from, to) arguments to standard forms for passing
+# to C++ routines
+prepare_graph <- function (graph, from, to)
+{
+    if ("flow" %in% names (graph))
+        warning ("graph already has a 'flow' column; ",
+                  "this will be overwritten")
+
+    gr_cols <- dodgr_graph_cols (graph)
+    vert_map <- make_vert_map (graph, gr_cols)
+
+    # change from and to just to check conformity
+    tp <- attr (graph, "turn_penalty")
+    tp <- ifelse (is.null (tp), 0, tp)
+
+    # remove any routing points not in edge start nodes:
+    from <- nodes_arg_to_pts (from, graph)
+    if (methods::is (graph, "dodgr_streetnet_sc") & tp > 0)
+        from <- remap_verts_with_turn_penalty (graph, from, from = TRUE)
+    from <- from [from %in% graph [[gr_cols$from]] ]
+    index_id <- get_index_id_cols (graph, gr_cols, vert_map, from)
+    from_index <- index_id$index - 1 # 0-based
+
+    to_index <- NULL
+    if (!missing (to))
+    {
+        # remove any routing points not in edge end nodes:
+        to <- nodes_arg_to_pts (to, graph)
+        if (methods::is (graph, "dodgr_streetnet_sc") & tp > 0)
+            to <- remap_verts_with_turn_penalty (graph, to, from = FALSE)
+        to <- to [to %in% graph [[gr_cols$to]] ]
+        index_id <- get_index_id_cols (graph, gr_cols, vert_map, to)
+        to_index <- index_id$index - 1 # 0-based
+    }
+
+    graph2 <- convert_graph (graph, gr_cols)
+
+    list (graph = graph2, vert_map = vert_map,
+          from_index = from_index, to_index = to_index)
+}
+
+get_random_prefix <- function (prefix = "flow", n = 5)
+{
+    charvec <- c (letters, LETTERS, 0:9)
+    rand_prefix <- paste0 (sample (charvec, n, replace = TRUE), collapse = "")
+    file.path (tempdir (), paste0 (prefix, "_", rand_prefix))
+}
+
+nodes_arg_to_pts <- function (nodes, graph)
+{
+    if (!is.matrix (nodes))
+        nodes <- as.matrix (nodes)
+    if (ncol (nodes) == 2)
+    {
+        verts <- dodgr_vertices (graph)
+        nodes <- verts$id [match_pts_to_graph (verts, nodes)]
+    }
+    return (nodes)
+}
+
+
+# keep from and to routing points in contracted graph
+contract_graph_with_pts <- function (graph, from, to)
+{
+    pts <- NULL
+    if (!missing (from))
+        pts <- c (pts, from)
+    if (!missing (to))
+        pts <- c (pts, to)
+    dodgr_contract_graph (graph, unique (pts))
+}
