@@ -163,6 +163,10 @@ find_d_col <- function (graph)
                     tolower (substring (names (graph), 1, 2)) != "dz" &
                     tolower (substring (names (graph), 2, 2)) != "w" &
                     tolower (substring (names (graph), 2, 2)) != "_")
+    if (length (d_col) != 1)
+    {
+        d_col <- which (tolower (substring (names (graph), 1, 2)) == "di")
+    }
     if (length (d_col) != 1)
         stop ("Unable to determine distance column in graph")
     return (d_col)
@@ -276,7 +280,8 @@ match_pts_to_graph <- function (verts, xy, connected = FALSE)
         if (!is (xy$geometry, "sfc_POINT"))
             stop ("xy$geometry must be a collection of sfc_POINT objects")
         xy <- unlist (lapply (xy$geometry, as.numeric)) %>%
-            matrix (nrow = 2) %>% t ()
+            matrix (nrow = 2) %>%
+            t ()
         xy <- data.frame (x = xy [, 1], y = xy [, 2])
     } else
     {
@@ -298,15 +303,15 @@ match_points_to_graph <- function (verts, xy, connected = FALSE)
     match_pts_to_graph (verts, xy, connected = connected)
 }
 
-# vertices randomly selected from a graph without turn penalties may be submitted
-# to functions along with the corresponding graph with turn angles. The latter
-# version appends vertex IDs with "_start" and "_end" for the starts and ends of
-# compound turn angle junctions. This function finds any instances of `pts` that
-# map on to these, and appends the appropriate suffix so these points can be
-# used in routines with the turn-penalty graph.
+# vertices randomly selected from a graph without turn penalties may be
+# submitted to functions along with the corresponding graph with turn angles.
+# The latter version appends vertex IDs with "_start" and "_end" for the starts
+# and ends of compound turn angle junctions. This function finds any instances
+# of `pts` that map on to these, and appends the appropriate suffix so these
+# points can be used in routines with the turn-penalty graph.
 remap_verts_with_turn_penalty <- function (graph, pts, from = TRUE)
 {
-    if (!is (graph, "dodgr_streetnet_sc"))
+    if (!methods::is (graph, "dodgr_streetnet_sc"))
         stop ("vertices with turn angles can only be re-mapped for ",   # nocov
               "street networks obtained via 'dodgr_streetnet_sc' -> ",  # nocov
               "'weight_streetnet'")                                     # nocov
