@@ -5,7 +5,6 @@
 #include <vector>
 #include <map>
 #include <limits>
-#include <random>
 #include <string> // stoi
 #include <cmath> // round
 #include <math.h> // isnan
@@ -179,8 +178,6 @@ Rcpp::StringVector rcpp_sample_graph (Rcpp::DataFrame graph,
 
 namespace graph_contract {
 
-edge_id_t get_new_edge_id (edge_map_t &edge_map, std::mt19937 &rng);
-
 void get_to_from (const edge_map_t &edge_map,
         const std::unordered_set <edge_id_t> &edges,
         const std::vector <vertex_id_t> &two_nbs,
